size = [10.0, 10.0, 5.0];

box_and_lid(size) {
    translate([-size[0] / 2, -size[1] / 2, 0.0])
    cube(size);
}


module box_and_lid(size_inner) {
    dovetail_angle          = 60;
    dovetail_clearance_side = 0.1;
    dovetail_clearance_top  = 0.2;
    wall_thickness_min      = 1.5;

    dovetail_height = wall_thickness_min;
    dovetail_offset_min =
        wall_thickness_min
        + dovetail_clearance_side
        - dovetail_clearance_top / tan(dovetail_angle);
    dovetail_offset_max =
        dovetail_offset_min
        + tan(90 - dovetail_angle) * (dovetail_height + dovetail_clearance_top);
    wall_thickness_outer =
        dovetail_offset_min
        + wall_thickness_min;

    width_outer  = size_inner[0] + 2 * wall_thickness_outer;
    length_outer = size_inner[1] + 2 * wall_thickness_outer;
    height_outer = size_inner[2] + 2 * wall_thickness_outer + dovetail_height;

    box();

    translate([
        wall_thickness_outer + size_inner[0] / 2,
        wall_thickness_outer + size_inner[1] / 2,
        wall_thickness_outer
    ])
    children();

    translate([width_outer + 10.0, 0.0, 0.0])
    lid(0.2);

    module box() {
        difference() {
            cube([width_outer, length_outer, height_outer]);

            translate([
                wall_thickness_outer,
                wall_thickness_outer,
                wall_thickness_outer
            ])
            // Need to subtract slightly more than inner size in height.
            // Otherwise OpenSCAD will leave a thing film of material, for some
            // reason.
            cube([size_inner[0], size_inner[1], size_inner[2] + 0.1]);

            translate([0.0, 0.0, height_outer])
            mirror([0, 0, 1])
            lid(0.0);
        }
    }

    module lid(clearance) {
        difference() {
            union() {
                body();
                front();
                dovetail(clearance);
            }
            knob();
        }

        module body() {
            cube([
                width_outer,
                length_outer,
                wall_thickness_outer,
            ]);
        }

        module front() {
            translate([0.0, 0.0, wall_thickness_outer])
            cube([width_outer, wall_thickness_outer, dovetail_height]);
        }

        module dovetail(clearance) {
            left_top     = -width_outer / 2 + dovetail_offset_max + clearance;
            left_bottom  = -width_outer / 2 + dovetail_offset_min + clearance;
            right_bottom =  width_outer / 2 - dovetail_offset_min - clearance;
            right_top    =  width_outer / 2 - dovetail_offset_max - clearance;

            translate([
                width_outer / 2,
                wall_thickness_outer,
                wall_thickness_outer
            ])
            rotate([-90, 0, 0])
            linear_extrude(size_inner[1])
            polygon([
                [left_top,     0.0],
                [left_bottom,  -dovetail_height],
                [right_bottom, -dovetail_height],
                [right_top,    0.0]
            ]);
        }

        module knob() {
            knob_width  = wall_thickness_min;
            knob_height = 0.2;

            translate([
                width_outer / 2,
                length_outer - wall_thickness_outer / 2,
                wall_thickness_outer
            ])
            union() {
                inner();

                mirror([1, 0, 0])
                inner();
            }

            module inner() {
                translate([width_outer / 2 - knob_width, 0.0, 0.0])
                rotate([0, 90, 0])
                linear_extrude(knob_width)
                scale([1.0, 2.0, 0.0])
                circle(r = knob_height, $fn = 360);
            }
        }
    }
}
